/*
 * $Id: ostool.c 403 2008-05-19 21:31:37Z tobias.pape $
 *
Copyright (c) 2007 Michael Haupt, Tobias Pape
Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
http://www.hpi.uni-potsdam.de/swa/

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
  */

/* The ostool is a small tool for generating platform-specific command line
 * switches etc.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(__GNUC__)
#if defined(__APPLE__) && defined(__MACH__)
#	define executable_extension ""
#	define LF_program ""
#	define LF_shared " -fPIC -mmacosx-version-min=10.4" \
                     " -undefined dynamic_lookup -dynamiclib" \
                     " -Wl,-single_module -Wl,-Y,1455 "
#	define LF_shared_name  " -install_name "
#   define LF_combine_objects ""
#   define LF_combine_libs ""

#elif  defined(linux) || defined(__FreeBSD__) || defined(__NetBSD__)
#	define executable_extension ""
#	define LF_program " -Wl,--export-dynamic "
#	define LF_shared " -fPIC -shared"
#	define LF_shared_name  " -Wl,-soname,"
#   define LF_combine_objects ""
#   define LF_combine_libs ""

#elif defined(WIN32) || defined(__CYGWIN32__)
#	define executable_extension ".exe"
#	define LF_program ""
#	define LF_shared " -shared" \
                     " -Wl,--export-all-symbols" \
                     " -Wl,--enable-auto-import"
#   define LF_import_name " -Wl,--out-implib="
#   define LF_import_ext ".lib"
#   define LF_combine_objects " -Wl,--whole-archive "
#   define LF_combine_libs " -Wl,--no-whole-archive "

#elif defined(sun)
#	define executable_extension ""
#	define LF_program " -Wl,-dy "
#	define LF_shared " -fPIC -shared -Wl,-G "
#   define LF_combine_objects ""
#   define LF_combine_libs ""

#else
#    error "no currently supported platform"
/*# APPLE/linux/bsd/win32/sun */
#endif
#else
#error please use gcc
/*#  defined(__GNUC__) */
#endif

#define shared_extension ".csp"

void print_header(void) {
    printf(
    "#ifndef PLATFORM_H_\n" 
    "#define PLATFORM_H_\n" 
    "/* this file is autogenerated. DO NOT EDIT */\n" 
#if (defined(__APPLE__) && defined(__MACH__)) || defined(linux) || \
    defined(__FreeBSD__) || defined(__NetBSD__) || defined(sun)
    "#define path_separator \":\"\n" 
    "#define file_separator \"/\"\n" 
	"#define CSOM_UNIX\n"
#elif defined(WIN32) || defined(__CYGWIN32__)
    "#define path_separator \";\"\n" 
    "#define file_separator \"\\\\\"\n"
	"#define CSOM_WIN\n"
#else
    "#error \"no currently supported platform\"\n" 
#endif APPLE/linux/bsd/win32/sun

#if defined(sun)
/* mingw should follow */
    "#define NEED_STRSEP\n\n"
#endif
    
    "#define shared_extension \"" shared_extension "\"\n"
    "\n// EOF \n#endif PLATFORM_H_\n");
}

int main(int argc, char **argv) {

#define OSTOOL_USAGE " usage: %s {[sxl] <filename> | [il] | c [ol]}\n\n"
    
    if(argc <2) {
        printf(OSTOOL_USAGE, argv[0]);
        exit(-1);
    }
    
    switch(*argv[1]) {
        case 'l':
            if(argc < 3)
                printf("%s", LF_program);
            else {
#if defined(LF_shared_name)
                printf("%s%s%s%s",LF_shared, 
                       LF_shared_name, argv[2], shared_extension);
#elif defined(LF_import_name)
                printf("%s%s%s%s",LF_shared, 
                       LF_import_name, argv[2], LF_import_ext);
#else
                printf("%s", LF_shared);
/*# !defined(LF_shared_name) */
#endif 
            }
            break;
        case 's':
            printf("%s%s", argv[2], shared_extension);
            break;
        case 'x':
            printf("%s%s", argv[2], executable_extension);
            break;
        case 'i':
            print_header();
            break;
        case 'c':
            switch(*argv[2]) {
                case 'o':
                    printf("%s", LF_combine_objects);
                    break;
                case 'l':
                    printf("%s", LF_combine_libs);
                    break;
                default: break;
            }
            break;
        default:
            printf(OSTOOL_USAGE, argv[0]);
            exit(-1);
    }
    return 0;
}
